(defconst ee-VERSION "v1.42" "The version of Easy-edit being run")

;
; first check for some conditions which are deadly to ee
;

(if (not (null window-system))
    (progn
      (message "Easyedit cannot run with Emacs in windows mode" (ding))
      (sleep-for 3)
      (message "Please run easyedit from an xterm or other terminal emulation window")
      (sleep-for 5)
      (kill-emacs)))

(if (null buffer-file-name)
	(progn
      (message "You must edit a named file to run ee" (ding))
      (sleep-for 4)
      (kill-emacs)))

;                    Easy-Edit  v1.42       GNU Emacs configuration
;			Dec-20-1993
;
;
; Easy-edit novice screen editor for Unix systems with GNU emacs
;
; Copyright (C) 1992     Larry Broda
;                      
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This program 
;    is an extension of GNU emacs; See the GNU General Public License 
;    for more details.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the same terms as the GNU General Public License as 
;    published by the Free Software Foundation; either version 1, or 
;    (at your option) any later version.
;
;
;
;	Easy-edit v1.42
;       By Larry Broda  
;	Department of Psychology
;	University of Illinois at Urbana-Champaign
;       Email: lbroda@s.psych.uiuc.edu
;	4/13/92 
;
;    GNU emacs information:
;    contact:
;    Free Software Foundation
;    675 Massachusetts Avenu
;    Cambridge, MA 02139
;    USA

;;;
;;; Our variables and constants
;;;

(defconst ee-com-map nil
  "Keymap for easyedit")

(defvar ee-added-welcome "" "Text to add to welcome message on startup")
(defvar ee-EWIN )
(defvar ee-BLOCK-MARK 0)
(defvar ee-BLOCK-SAVED nil)
(defvar ee-RES_CMD "" "Shell command for restarting ee after suspending")
(defvar EASYEDIT-INSERT 1 "Flag for insert/overwrite mode")
(defvar ee-LAST-SEARCHSTRING ""  "Search string from last search")
(defvar ee-MODE)
(defvar ee-FILE_NAME nil "Name of file being edited")
(defvar ee-RESUME_MSG nil "Message issued in response to a suspend")
(defvar EEHELPF)
(defvar EASYEDIT_MANUAL)
(defvar ALTERNATE_HELP)
(defvar ee-help-mode-line)
(defvar ee-HWIN)
(defvar ee-HWSIZE)
(defvar char)
(defvar command)
(defvar ssprompt)
(defvar sprompt)
(defvar sstring)
(defvar delflg)
(defvar rstring)
(defvar SRFLG)
(defvar temp)
(defvar ifile)
(defvar achar)
(defvar IMSG)
(defvar savename "")
(defvar SAVEFL)
(defvar gamsg)
(defvar ldir)
(defvar lcomm)

;
; Function to give an error message for using an key out of context
;

(defun ee-not-here ()
  "That key unbound at moment"
  (interactive)
  (ding))

;
; Function to give an error message for using an key out of context
;
(defun ee-notin-mini ()
  "That key cannot be used in mini buffer"
  (interactive)
  (ding))

;
; 
;
(defun ee-warn-escapeuse ()
  "function to give an error for ESC ESC"
  (interactive)
  (message "For ESC commands, press and release ESC, then press the next key" (ding))
  (sleep-for 2))

(defun ee-nofun ()
  "function to give an error for an unbound key"
  (interactive)
  (message "Not currently an easy-edit command" (ding))
  (sleep-for 2))

(defun ee-quit ()
  "Exit without saving"
  (interactive)	
  (progn
    (if (buffer-modified-p (get-buffer "EDIT"))
	(progn
	  (cond 
	   ((yes-or-no-p "Really quit and cancel changes? " ) 
	    (message "Cancelling changes!")
	    (sleep-for 1)
	    (discard-input)
	    (kill-emacs))
	   ((message "resume editing"))))
      (message "No changes to cancel...")
      (sleep-for 1)
      (discard-input)
      (kill-emacs))))

(defun ee-kill-to-eof ()
  "Delete from point to End of buffer"
  (interactive)
  (delete-region (point)(point-max)))

(defun ee-fill-to-eof ()
  "fill paragraphs from point to End of buffer"
  (interactive)
  (end-of-buffer)
  (fill-region (point) (mark))
  (exchange-point-and-mark))

(defun ee-move-block ()
  "Demark a block of text and copy or move it"
  (interactive)

  (if (> ee-BLOCK-MARK 0)
	 (progn 

	   (let ((imsg "
		 Type  m  to move the block (delete original)
		 Type  c  to copy the block (leave original)

                 Type ESC +  when ready to insert the block at cursor

                 ... you may continue to insert the block repeatedly
                     until you mark another block.")
		 (iprmpt "Choose m or c, please: "))

	     (setq delflg (eq (ee-choosefrom-list imsg iprmpt '(?c ?m)) ?m)))

	   (copy-to-register ?b ee-BLOCK-MARK (point) delflg)
	   (setq ee-BLOCK-MARK 0)
	   (message "type ESC + to insert block at cursor location")
	   (setq ee-BLOCK-SAVED 'true)
	   (ding))

    (setq ee-BLOCK-MARK (point))
    (setq ee-BLOCK-SAVED 'nil)
    (message "Move cursor to other end of text block and press ESC / again")
    (ding)
    (sleep-for 2)))

;
; Code from GNU Emacs: ee-query-replace and ee-perform-replace are modifications by
; Larry Broda, Dept. of Psychology/University of Illinois at C-U, of the
; functions query-replace and perform-replace from the GNU Emacs distribution, and are subject
; to all terms of the GNU Emacs Public License.
;

(defun ee-query-replace (from-string to-string &optional arg)
  "\
Replace some occurrences of FROM-STRING with TO-STRING.
As each match is found, the user must type a character saying
what to do with it.  
Preserves case in each replacement if  case-replace  and  case-fold-search
are non-nil and FROM-STRING has no uppercase letters.
Third arg DELIMITED (prefix arg if interactive) non-nil means replace
only matches surrounded by word boundaries."
  (interactive "sQuery replace: \nsQuery replace %s with: \nP")
  (ee-perform-replace from-string to-string t nil arg)
  (message "Done"))

(defun ee-perform-replace (from-string to-string
		        query-flag regexp-flag delimited-flag)
  (let ((nocasify (not (and case-fold-search case-replace
			    (string-equal from-string
					  (downcase from-string)))))
	(literal (not regexp-flag))
	(search-function (if regexp-flag 're-search-forward 'search-forward))
	(search-string from-string)
	(keep-going t)
	(lastrepl nil))			;Position after last match considered.
    (if delimited-flag
	(setq search-function 're-search-forward
	      search-string (concat "\\b"
				    (if regexp-flag from-string
				      (regexp-quote from-string))
				    "\\b")))
    (push-mark)
    (push-mark)
    (while (and keep-going
		(not (eobp))
		(progn
		 (set-mark (point))
		 (funcall search-function search-string nil t)))
      ;; Don't replace the null string 
      ;; right after end of previous replacement.
      (if (eq lastrepl (point))
	  (forward-char 1)
	(undo-boundary)
	(if (not query-flag)
	    (replace-match to-string nocasify literal)
	  (let (done replaced)
	    (while (not done)
	      ;; Preserve the match data.  Process filters and sentinels
	      ;; could run inside read-char..
		 
		(message "Query replacing %s with %s: " from-string to-string)
		(setq char (read-char))

	      (cond ((or (= char ?\e)
			 (= char ?q))
		     (setq keep-going nil)
		     (setq done t))
		    ((= char ?y)
		     (or replaced
			 (replace-match to-string nocasify literal))
		     (setq done t))
		    ((= char ?\.)
		     (or replaced
			 (replace-match to-string nocasify literal))
		     (setq keep-going nil)
		     (setq done t))
		    ((= char ?!)
		     (or replaced
			 (replace-match to-string nocasify literal))
		     (setq done t query-flag nil))
		    ((= char ?n)
		     (setq done t))
		    ((= char ?\C-l)
		     (recenter nil))
		    (t
		     (message "Not a choice")
		     (ding)
		     (sleep-for 2))))))
	(setq lastrepl (point))))
    (pop-mark)
    keep-going))

(defun ee-search-replace ()
  "Do a text search/replace"
  (interactive)
  (setq sprompt (concat "Replace-> [" ee-LAST-SEARCHSTRING "]: "))
  (setq sstring (read-string sprompt))
  (if (> (length sstring) 0)
      (setq ee-LAST-SEARCHSTRING sstring)
    (setq sstring ee-LAST-SEARCHSTRING))
  
    (if (> (length sstring) 0)
	(progn
	  (setq rstring (read-string "Replace with-> "))
	  (save-window-excursion
	    (with-output-to-temp-buffer "*INFORMATION*"
	      (save-excursion
		(set-buffer "*INFORMATION*")
		(erase-buffer))
	      (princ "             Search/Replace\n\n")
	      (princ (concat "Replace all occurrences of " sstring " with " rstring "\n"))
	      (princ "from the current position to the end of file.\n")
	      (princ "(use confirm if you don't want to replace every occurrence)\n\n")
	      (princ "Type y for confirm (will ask before doing each replacement)\n")
	      (princ "Type n to do all replacements without confirming\n"))
	    (setq SRFLG ?x)
	    (while (not (ee-member SRFLG '(?y ?n)))
	      (progn
		(let ((cursor-in-echo-area t))
		  (message "Confirm replacements (y or n)? ")
		  (ding)
		  (setq SRFLG (downcase (read-char))))
		(let ((cursor-in-echo-area nil))
		  (cond
		 ((eq SRFLG ?y)
		  (with-output-to-temp-buffer "*INFORMATION*"
		    (save-excursion
		      (set-buffer "*INFORMATION*")
		      (erase-buffer))
		    (princ "Choices for answering replace query: \n\n")
		    (princ "(press ESC or q to stop the search/replace at any time)\n\n")
		    (princ "  y     - do the replacement and continue searching\n")
		    (princ "  n     - don't replace, but continue searching\n")
		    (princ "  .     - Replace this occurrence and stop\n")
		    (princ "  q     - Don't replace, and stop\n")
		    (princ "  !     - replace all occurrences from here on without confirming\n"))
		  (ee-query-replace sstring rstring nil))
		 ((eq SRFLG ?n)  (replace-string sstring rstring))
		 (t  (message "Type y for confirm, n for no confirm, please")
		     (sleep-for 2))))))))))

(defun ee-insert-block ()
  "Insert block"
  (interactive)

  (if (null ee-BLOCK-SAVED )
      (progn
        (message "No block has been marked and saved with ESC /")
        (ding))
    (insert-register ?b)))
	

(defun ee-recover ()
  "recover from an Emacs auto-save file"
  (interactive)
  (ee-recover-file ee-FILE_NAME)
  (auto-save-mode 1))
;;;;  (easyedit-mode))

;
; following function derived from GNU Emacs recover-file function, by
; Richard Stallman
;
(defun ee-recover-file (file)
  "Visit file FILE, but get contents from its last auto-save file."
  (interactive "FRecover file: ")
  (setq file (expand-file-name file))
  (if (auto-save-file-name-p file) (error "%s is an auto-save file" file))
  (let ((file-name (let ((buffer-file-name file))
             (make-auto-save-file-name))))
    (cond ((not (file-newer-than-file-p file-name file))
       (error "Auto-save file %s not current" file-name))
       ((yes-or-no-p "Recover previous unsaved edits? " ) 
        (switch-to-buffer (find-file-noselect file t))
        (let ((buffer-read-only nil))
         (erase-buffer)
         (insert-file-contents file-name nil))
        (after-find-file nil))
		(t (error "Recovery cancelled.")))
  (setq buffer-auto-save-file-name nil)))
  
 
(defun ee-check-recovery-file (file)
  "Print a nicer message if the save file is current"
  (interactive)
  (setq file (expand-file-name file))
  (if (auto-save-file-name-p file) (error "%s is an auto-save file" file))
  (let ((file-name (let ((buffer-file-name file))
             (make-auto-save-file-name))))
    (cond ((file-newer-than-file-p file-name file)
           (message "Found recovery file from unsaved edit; consider ESC R to recover.")
            (ding)
            (sleep-for 4)))))

(defun ee-suspend-edit ()
  "suspend"
  (interactive)
    (suspend-emacs ee-RESUME_MSG))

(defun ee-create-help ()
  "Create the help window"
   (progn
    (setq EEHELPF
	  (cond
	   ((setq temp (getenv "EASYEDIT_HELP")) temp)
	   (t "/usr/local/lib/easyedit/HELP-WINDOW")))
    (setq ALTERNATE_HELP (concat EEHELPF "-ADVANCED"))
    (setq EASYEDIT_MANUAL
	  (cond
	   ((setq temp (getenv "EASYEDIT_MANUAL")) temp)
	   (t "/usr/local/lib/easyedit/HELP-MANUAL")))
    (if (string-equal (getenv "HELP_LEVEL") "advanced")
	(progn
	  (setq temp ALTERNATE_HELP)
	  (setq ALTERNATE_HELP EEHELPF)
	  (setq EEHELPF temp)))
    (if (file-exists-p EEHELPF)
	(progn
	  (setq ee-HWIN (split-window nil nil nil))
	  (select-window ee-HWIN)
	  (switch-to-buffer "HELP")
	  (insert-file-contents EEHELPF)
	  (setq buffer-read-only t)

; 7/2/92
	  (setq mode-line-format ee-help-mode-line)
	  (select-window ee-EWIN))

      (message "HELP FILE NOT FOUND!")
      (ding)
      (sleep-for 3)
      (discard-input)
      (kill-emacs))))

(defun ee-switch-help ()
  "toggle between the advanced and basic help files"
  (interactive)

  (if (not (file-exists-p ALTERNATE_HELP))
      (progn
	(message "Alternate help file not found!")
	(ding))
    (define-key ee-com-map "\e>" 'ee-not-manual)
    (define-key ee-com-map "\e<" 'ee-not-manual)
    (set-buffer "HELP")
    (setq buffer-read-only nil)
    (erase-buffer)
    (insert-file-contents ALTERNATE_HELP)
    (setq buffer-read-only t)
; 7/2/92
    (setq mode-line-format ee-help-mode-line)
    (set-buffer "EDIT")
    (setq temp ALTERNATE_HELP)
    (setq ALTERNATE_HELP EEHELPF)
    (setq EEHELPF temp)))

(defun ee-manual ()
  "replace HELP with MANUAL display"
  (interactive)
  (if (not (file-exists-p EASYEDIT_MANUAL))
      (progn
	(message "Easy-edit manual file not found!")
	(ding))
    (set-buffer "HELP")
    (let ((buffer-read-only nil))
      (erase-buffer)
      (insert-file-contents EASYEDIT_MANUAL))
; 7/2/93

    (setq mode-line-format (list "Manual: ESC f  forward --- ESC b  backward --- ESC H  restore HELP"))

    (set-buffer "EDIT")
    (define-key ee-com-map "\e>" 'ee-next-subject)
    (define-key ee-com-map "\e<" 'ee-previous-subject)
    (message "Easy-edit Manual")))

(defun ee-insert-file ()
  "Insert file"
  (interactive)
  (setq ifile (read-string "File to insert: "))
  (if (> (length ifile) 0) 
      (let ((default-directory "./"))
	(insert-file-contents ifile))))

(defun ee-search-forward ()
  "search forward"
  (interactive)
  (setq sprompt (concat "Find-> [" ee-LAST-SEARCHSTRING "]: "))
  (setq sstring (read-string sprompt))
  (if (> (length sstring) 0) (setq ee-LAST-SEARCHSTRING sstring)) 
  (search-forward ee-LAST-SEARCHSTRING))

(defun ee-search-backward ()
  "search forward"
  (interactive)
  (setq sprompt (concat "<-Find [" ee-LAST-SEARCHSTRING "]: "))
  (setq sstring (read-string sprompt))
  (if (> (length sstring) 0) (setq ee-LAST-SEARCHSTRING sstring)) 
  (search-backward ee-LAST-SEARCHSTRING))

(defun ee-upp-help ()
  "Scroll help window upwards"
  (interactive)
  (if (window-point ee-HWIN)
      (progn
	(select-window ee-HWIN)
	(setq ee-HWSIZE (- (window-height) 1))
	(select-window ee-EWIN)
	(scroll-other-window (* -1 ee-HWSIZE)))
    (message "No HELP window to scroll")
    (ding)
    (sleep-for 2)))

(defun ee-downp-help ()
  "Scroll help window upwards"
  (interactive)
  (if (window-point ee-HWIN)
      (progn
	(select-window ee-HWIN)
	(setq ee-HWSIZE (- (window-height) 1))
	(select-window ee-EWIN)
	(scroll-other-window ee-HWSIZE))
    (message "No HELP window to scroll")
    (ding)
    (sleep-for 2)))


(defun ee-upl-help ()
  "Scroll help window upwards"
  (interactive)
  (if (window-point ee-HWIN)
      (scroll-other-window -1)
    (message "No HELP window to scroll")
    (ding)
    (sleep-for 2)))

(defun ee-downl-help ()
  "Scroll help window upwards"
  (interactive)
  (if (window-point ee-HWIN)
      (scroll-other-window 1)
    (message "No HELP window to scroll")
    (ding)
    (sleep-for 2)))


(defun ee-kill-line (cnt)
  "Delete a line"
  (interactive "P")
  (progn 
    (beginning-of-line)
    (kill-line (if (eq cnt nil) 1 cnt))))

(defun ee-restore-help ()
  "restore a lost help window"
  (interactive)
  (progn
    (define-key ee-com-map "\e>" 'ee-not-manual)
    (define-key ee-com-map "\e<" 'ee-not-manual)
    (switch-to-buffer "EDIT")
    (delete-other-windows)
    (setq ee-HWIN (split-window nil nil nil))
    (select-window ee-HWIN)
    (switch-to-buffer "HELP")
    (let ((buffer-read-only nil))
      (erase-buffer)
      (insert-file-contents EEHELPF))
   ; 7/2/92
    (setq mode-line-format ee-help-mode-line)
    (select-window ee-EWIN)))

(defun ee-zap-help ()
  "Remove the HELP window"
  (interactive)
  (progn
    (delete-other-windows)
    (switch-to-buffer "EDIT")
    (message "To display HELP window: ESC H")))


  ;
  ;
  ; display the *INFORMATION* window with the mesg "mesg", for 
  ; duration seconds.  If duration =0, display it until a character is typed
  ; if cmd is non-nil, execute the function
  ;
(defun ee-INFO (mesg)
  (save-window-excursion
    (with-output-to-temp-buffer "*INFORMATION*"
      (save-excursion
	(set-buffer "*INFORMATION*")
	(erase-buffer)
	)
      (princ "\n\n")
      (princ mesg))
    (let ((cursor-in-echo-area t))
      (message "Type any character to continue: ")(read-char))
    ))


;
; this command displays "mesg" in a tien INFORMATION window,
; and prompts for a character which must be a member of choices, using
; the prompt string "prompt" to prompt for it
;
(defun ee-choosefrom-list (mesg prompt choices)

  (progn
      (save-window-excursion
	(with-output-to-temp-buffer "*INFORMATION*"
	  (save-excursion
	    (set-buffer "*INFORMATION*")
	    (erase-buffer)
	    )
	  (princ "\n\n")
	  (princ mesg))
	(let ((cursor-in-echo-area t))
	  (setq achar '(x))
	  (while (not (ee-member achar choices))
	    (message prompt)
	    (ding)
	    (setq achar (downcase (read-char)))))
	)
      achar))

(defun ee-member (item list)
  "is item in list?"
  (cond
   ((null list)  nil)
   ((eq item (car list))   list)
   (t        (ee-member item (cdr list)))))

(defun ee-status ()
  "status "
  (interactive)
  (progn
    (setq IMSG
	  (concat (current-time-string) "
File  : " buffer-file-name "
status: " (cond ((buffer-modified-p (get-buffer "EDIT")) "Modified")
		(t "Not modified")) " 
size  : " (buffer-size) " characters 

Easy-edit running under Emacs version: " emacs-version " 

(type ESC @ for easy-edit license information)"))
    (ee-INFO IMSG)))

(defun ee-get-arg (mesg prompt)
  (let (char)
      (save-window-excursion
(with-output-to-temp-buffer "*INFORMATION*"
  (save-excursion
    (set-buffer "*INFORMATION*")
    (erase-buffer)
    )
  (princ mesg))
(let ((cursor-in-echo-area t))
  (read-string prompt))
)))

(defun ee-save-exit ()
  "save file and exit"
  (interactive)
  (progn
    (if (buffer-modified-p (get-buffer "EDIT"))
	(message (concat "Saving " ee-FILE_NAME))
      (message (concat ee-FILE_NAME " not modified")))
    (sleep-for 1)      
    (save-buffer)
    (discard-input)
    (kill-emacs)))

(defun ee-save-noexit ()
  "save file but do not exit"
  (interactive)
  (progn
    (if (buffer-modified-p (get-buffer "EDIT"))
	(message (concat "Saving " ee-FILE_NAME))
      (message (concat ee-FILE_NAME " not modified")))
    (sleep-for 1)      
    (save-buffer)))

(defun ee-saveto-file ()
  "save file and exit"
  (interactive)
  (let ((default-directory "./"))
    (progn
      (set-buffer "EDIT")
      
      (setq sprompt "Save to what filename: ")
      (setq savename (read-string sprompt))
      (if (> (length savename) 0) 
	  (progn
	    (if (file-writable-p savename)
		(progn
		  (setq SAVEFL
			(if (file-exists-p savename)
			    (yes-or-no-p 
			     (concat savename " exists!  Overwrite it? "))
			  t))
		  (if SAVEFL
		      (progn


;;;			(write-file savename)
			(setq buffer-file-name savename)
			(clear-visited-file-modtime)
			(set-buffer-modified-p t)
			(message 
			 (concat  " saving  " savename "; leaving "
				 ee-FILE_NAME " unchanged"))
			(sleep-for 2)
			(save-buffer)
;;;			(message 
;;;			 (concat savename " saved;  " 
;;;				 ee-FILE_NAME " unchanged"))
;;;			(sleep-for 2)
;;;			(discard-input)
			(kill-emacs))))
	      (message (concat "Cannot create/write to " savename))))))))


(defun ee-grow-EDIT ()
  "Enlarge EDIT window"
  (interactive)
  (cond 
   ((window-point ee-HWIN) 
    (progn
      (enlarge-window 1)
      (if (not (window-point ee-HWIN))
	  (message "To restore HELP window: ESC H"))))))


(defun ee-next-subject ()
  "Move to the next subject in the manual"
  (interactive)
  (if (window-point ee-HWIN)
      (progn
	(select-window ee-HWIN)
	(forward-line)
	(if (not (re-search-forward "^=" nil t))
	    (progn
	      (beginning-of-buffer)
	      (re-search-forward "^=" nil t)))
	(recenter 1)
	(select-window ee-EWIN))))
 
(defun ee-previous-subject ()
  "Move to the next subject in the manual"
  (interactive)
  (if (window-point ee-HWIN)
      (progn
	(select-window ee-HWIN)
	(forward-line -1)
	(if (not (re-search-backward "^=" nil t))
	    (progn
	      (end-of-buffer)
	      (re-search-backward "^=" nil t)))
	(recenter 1)
	(select-window ee-EWIN))))

(defun ee-toggle-insert ()
  "toggle insert/overwrite modes"
  (interactive)
   (cond 
    ((= EASYEDIT-INSERT 1) 
     (overwrite-mode 1)
     (setq EASYEDIT-INSERT 0)
     (setq ee-MODE "  *Overwrite*"))
    (t   (overwrite-mode 0)
	 (setq EASYEDIT-INSERT 1)
	 (setq ee-MODE "  *Insert*"))))

; ; elm easy editor setup

(defun easyedit-mode ()
  "set easyedit mode"
  (if (eq major-mode 'easyedit-mode)
      (message "already in easyedit-mode" (ding))
    (setq mode-name "easy-edit")
    (setq major-mode 'easyedit-mode)
    (use-local-map ee-com-map)
    (put 'switch-to-buffer 'disabled t)
    (put 'switch-to-buffer-other-window 'disabled t)
    (put 'other-window 'disabled t)
    (put 'kill-region 'disabled t)
    (put 'execute-extended-command 'disabled t)
    (setq window-min-height 1)
    (setq fill-column 72)
    (setq ee-MODE "  *Insert*")
    (setq make-backup-files nil)
    (setq require-final-newline t)
    (auto-fill-mode 1)

    (if (setq ee-RES_CMD (getenv "EE_RESUME_CMD"))
	(setq ee-RESUME_MSG (concat "echo Suspending easy-edit.... to resume type " ee-RES_CMD))
      (setq ee-RESUME_MSG (concat "echo Suspending easy-edit")))


  (setq minor-mode-alist '(overwrite-mode " *overwrite*"))
  (setq mode-line-buffer-identification (list "easy-edit: " buffer-file-name))
  (setq mode-line-format (list mode-line-buffer-identification "  " minor-mode-alist "  %p" "-%-")) ))

(if ee-com-map nil

  (setq ee-com-map 

      '(keymap
	(t . self-insert-command) 

       ;; ctrl-a - ctrl-z ascii 1-26 ;  

	(1 . beginning-of-line) (2 . backward-char) (3 . abort-recursive-edit)
	(4 . scroll-up) (5 . end-of-line) (6 . forward-char) (7 . forward-word)
	(8 . backward-delete-char-untabify) (9 . self-insert-command)
	(10 . next-line) (11 . previous-line) (12 . ee-not-here)  
	(13 . newline)	(14 . next-line) (15 . backward-word) 
	(16 . previous-line) (17 . ee-search-backward) 
	(18 . redraw-display) (19 . ee-search-forward)(20 . ee-status) 
	(21 . scroll-down) (22 . next-line) (23 . kill-word) 
	(24 . delete-char) (25 . yank)(26 . ee-suspend-edit) 

       ;; see ESC keymap below for 27

	(28 . ee-nofun)  ; C-\
	(29 . ee-nofun)	; C-] 
	(30 . ee-nofun)  ; C-^    
	(31 . ee-nofun)	; C-_   

	(127 . backward-delete-char-untabify) ; DEL 

; the ESC keymap   -- use above listings for ascii ref
	(27 keymap			; ESC map
	    (t . ee-nofun)
;; punctuation
	    (27 . ee-warn-escapeuse)
	    (33 . ee-unix-command)	; !
	    (34 . quoted-insert)        ; "
	    (40 . ee-search-forward)	; (
	    (41 . ee-search-backward) ; )
	    (43 . ee-insert-block)	; +
	    (47 . ee-move-block)	; / 
	    (48 . digit-argument) (49 . digit-argument) (50 . digit-argument)
	    (51 . digit-argument)(52 . digit-argument) (53 . digit-argument)
	    (54 . digit-argument) (55 . digit-argument) (56 . digit-argument)
	    (57 . digit-argument)
	    (60 . ee-not-manual)	; <
	    (62 . ee-not-manual)	; >
	    (64 . ee-license)		; @    65-90 are A-Z
; alphabetics
	    (65 . beginning-of-buffer)   (97 . beginning-of-buffer) ; A a
	    (66 . ee-upl-help)	    (98 . ee-upp-help) ; B b
	    (67 . ee-repeat-count)	    (99 . ee-repeat-count) ; C c
	    (68 . ee-list-dir)	            (100 . ee-list-dir)	; D d
	    (69 . end-of-buffer)	    (101 . end-of-buffer) ; E e     
	    (70 . ee-downl-help)	    (102 . ee-downp-help) ; F f    
	    (71 . goto-line)	          (103 . goto-line) ; G g    
	    (72 . ee-restore-help)      (104 . ee-restore-help) ; H h     
	    (73 . ee-insert-file)  	    (105 . ee-insert-file) ; I i 
	    (74 . ee-grow-EDIT)	    (106 . ee-grow-EDIT) ; J j    
	    (75 . ee-kill-to-eof)	    (107 . ee-kill-to-eof) ; K k    
	    (76 . ee-kill-line)  	    (108 . ee-kill-line) ; L l   
	    (77 . ee-manual)	    (109 . ee-manual) ; M m     
	    (78. ee-toggle-insert)	    (110 . ee-toggle-insert) ; N n 
; O keymap for certain arrow keys
	    (79 keymap		; O keymap
		(t . ee-nofun)
		(65 . previous-line) ; OA
		(66 . next-line)	; OB
		(67 . forward-char) ; OC
		(68 . backward-char)) ; OD
	    (111 . ee-nofun)		; 111 = o
	    (80 . ee-fill-to-eof) (112 . fill-paragraph) ; P p
	    (81 . ee-quit)        (113 . ee-quit) ; Q q           
	    (82 . ee-recover)     (114 . ee-recover) ; R r   
	    (83 . ee-save-exit)   (115 . ee-save-exit) ; S s       
	    (84 . ee-saveto-file) (116 . ee-saveto-file) ; T t
	    (85 . undo)           (117 . undo) ; U u 
	    (86 . ee-switch-help) (118 . ee-switch-help) ; V v 
	    (87 . ee-save-noexit) (119 . ee-save-noexit) ; W w   
	    (88 . kill-line)      (120 . kill-line) ; X x    
	    (89 . ee-search-replace) (121 . ee-search-replace) ;Y y 
	    (90 . ee-zap-help)	(122 . ee-zap-help) ; Z z
; [ keymap for some arrow keys
	    (91 keymap 
		(t . ee-nofun)
		(65 . previous-line) ; [A
		(66 . next-line)	; [B
		(67 . forward-char) ; [C
		(68 . backward-char)) ; [D

       ;; end of ESC keymap
       )))
)

; make sure possible minibuffer exits to emacs are closed
(setq minibuffer-local-map
      '(keymap 
	(t . self-insert-command)
	;; ctrl-a - ctrl-z ascii 1-26 ;  
	(1 . beginning-of-line) (2 . backward-char) 
	 (3 . abort-recursive-edit)
	 (4 . ee-notin-mini) (5 . end-of-line) (6 . forward-char) 
	 (7 . forward-word) (8 . backward-delete-char-untabify) 
	 (9 . self-insert-command) (10 . exit-minibuffer) 
	 (11 . ee-notin-mini) (12 . ee-notin-mini) 
	 (13 . exit-minibuffer) (14 . ee-notin-mini) (15 . backward-word) 
	 (16 . ee-notin-mini) (17 . ee-notin-mini)
	 (18 . redraw-display) (19 . ee-notin-mini)  (20 . ee-notin-mini)
	 (21 . ee-notin-mini)   (22 . ee-notin-mini) (23 . kill-word)
	 (24 . delete-char) (25 . yank) (26 . ee-suspend-edit) 
	 (28 . ee-notin-mini) (29 . ee-notin-mini) (20 . ee-notin-mini)
	 (31 . ee-notin-mini)
	 (127 . backward-delete-char-untabify) ; DEL 
	 (27 keymap
	     (t . ee-notin-mini)
	    (27 . ee-warn-escapeuse)
	    (34 . quoted-insert)        ; "
	     (68 . ee-nofun)	   (100 . ee-list-dir)	; D d
	     (76 . ee-kill-line)     (108 . ee-kill-line) ; L l   
	     (78. ee-toggle-insert)  (110 . ee-toggle-insert) ; N n 
	     (79 keymap		; O keymap
		 (t . ee-notin-mini)
		 (65 . previous-line) ; OA
		 (66 . next-line)	; OB
		 (67 . forward-char) ; OC
		 (68 . backward-char)) ; OD
	     (85 . undo)           (117 . undo) ; U u 
	     (88 . kill-line)      (120 . kill-line)) ; X x    
	 )
)


(defun ee-list-dir ()
  "List a directory in another window"
  (interactive)

  (let ((default-directory "./"))
    (setq gamsg "\n\n\nType a filename, a directory name, or a wildcard 
(e.g. t*   lists all files starting with t)
to list particular files or an entire directory.\n
Type a carriage return alone to list the current working directory")
(setq ldir (ee-get-arg gamsg "Dir or file(s) to list: ")) 
(save-window-excursion
  (if (eq 0 (length ldir))
      (setq ldir "."))
  
  (setq lcomm (concat "ls -l " ldir))
  (setq command (concat "(set noglob; /bin/echo command executed: " lcomm ");" lcomm))
  (with-output-to-temp-buffer "*Directory*"
    (message "Working...")
    (call-process "csh" nil standard-output nil "-c" command))
  (let ((cursor-in-echo-area t))
    (setq char ?b)
    (while (not (eq char ?r))
      (progn 
	(message "(f)orward  (b)ackwards  (r)esume editing: ")
	(setq char (downcase (read-char)))
	(other-window 1)
	(cond ((eq char ?f) 
	       (if (not (pos-visible-in-window-p (point-max))) 
		   (scroll-up)(ding)(message "At bottom")(sleep-for 2)))
	      ((eq char ?b) 
	       (if (not (pos-visible-in-window-p (point-min))) 
		   (scroll-down)(ding)(message "At top")(sleep-for 2)))
	      ((eq char ?r) nil)
	      (t   (message "Type f, b, or r please: ")
		   (sleep-for 2)
		   t))
	(other-window 1))) )))
)

(defun ee-license ()
  "License info"
  (interactive)
(save-window-excursion

  (with-output-to-temp-buffer "*LICENSE*"

    (princ " Easy-edit novice screen editor for Unix systems with GNU emacs\n")
    (princ "
     Easy-edit " ) (princ ee-VERSION)
    (princ "

     Copyright (C) 1992     Larry Broda
                            Department of Psychology
                            University of Illinois at Urbana-Champaign

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This program  
    is an extension of GNU emacs; See the GNU General Public License  
    for more details. 

    This program is free software; you can redistribute it and/or modify
    it under the same terms as the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.


       By Larry Broda  
       Department of Psychology
       University of Illinois at Urbana-Champaign
       Email: lbroda@s.psych.uiuc.edu
       4/13/92 

    for GNU emacs information contact:
    Free Software Foundation
    675 Massachusetts Avenu
    Cambridge, MA 02139
    USA
"))
  (let ((cursor-in-echo-area t))
    (setq char ?b)
    (while (not (eq char ?r))
      (progn 
	(message "(f)orward  (b)ackwards  (r)esume editing: ")
	(setq char (downcase (read-char)))
	(other-window 1)
	(cond ((eq char ?f) 
	       (if (not (pos-visible-in-window-p (point-max))) 
		   (scroll-up)(ding)(message "At bottom")(sleep-for 2)))
	      ((eq char ?b) 
	       (if (not (pos-visible-in-window-p (point-min))) 
		   (scroll-down)(ding)(message "At top")(sleep-for 2)))
	      ((eq char ?r) nil)
	      (t   (message "Type f, b, or r please: ")
		   (sleep-for 2)
		   t))
	(other-window 1)))))
)

(defun ee-repeat-count ()
  "Warn the user that this function is gone"
  (interactive nil)
  (message "'Repeat' is no longer \"ESC C\".  Use \"ESC repeat-count command\"" (ding)))


(defun ee-not-manual ()
  "Error for using manual commands outside manual"
  (interactive)
  (message "This command only valid if manual is displayed")
  (ding)
  (sleep-for 3))

(defun ee-unix-command ()
  "execute a Unix command"
  (interactive)

  (setq gamsg "Type a unix command.  The output will be displayed in
a special window, where you may:
1: Scroll the output listing forward and backward (f/b).
2: Insert the command output listing into your edit file at the current 
   cursor location (i).
NOTE:  The command can *NOT* be a command that requires further input by you;
Use ^Z to escape to a Unix prompt to perform interactive commands.
EXAMPLES of valid commands: date; cat <filename>; who, etc.
EXAMPLES of *INVALID* commands: Elm; easyedit; Mail, etc.
...or anything that requires additional input from you..")
  (let ((default-directory "./"))
    (setq command (ee-get-arg gamsg "Unix Command: ")) 
    (save-window-excursion
      (with-output-to-temp-buffer "*Unix*"
	(message (concat "executing " (concat command "...")))
	(call-process "csh" nil standard-output nil "-c" command))
      (let ((cursor-in-echo-area t))
	(setq char ?b)
	(while (not (eq char ?r))
	  (progn 
	    (message "(f)orward  (b)ackwards  (i)nsert output into file  (r)esume editing: ")
	    (setq char (downcase (read-char)))
	    (other-window 1)
	    (cond ((eq char ?f) 
		   (if (not (pos-visible-in-window-p (point-max))) 
		       (scroll-up)(ding)(message "At bottom")(sleep-for 2)))
		  ((eq char ?b) 
		   (if (not (pos-visible-in-window-p (point-min))) 
		       (scroll-down)(ding)(message "At top")(sleep-for 2)))
		  ((eq char ?r) nil)
		  ((eq char ?i) (progn (copy-to-register ?v (point-min)(point-max) nil)
				       (other-window 1)
				       (insert-register ?v)
				       (other-window 1)))
		  
		  (t   (message "Type f, b, i, or r please: ")
		       (sleep-for 2)
		       t))
	    (other-window 1))))))
  )


;
;
;  create the separate windows
;


(progn
; 7/2/93
  (setq default-mode-line-format (list
				  ""
				  "Easy-edit information window"
				  " -- "
				  mode-line-process
				  '(-3 . "%p")
				  "-%-"))

  (setq ee-FILE_NAME (file-name-nondirectory buffer-file-name))
  (ee-check-recovery-file ee-FILE_NAME)
  (rename-buffer "EDIT")

; 7/2/93
  (setq ee-help-mode-line (list "scrolling HELP window:   ESC f  page forward --- ESC b   page backward " "-%-"))

  (setq ee-EWIN (selected-window))
  (setq blink-matching-paren nil)
  (ee-create-help)
  (easyedit-mode)
  (if (getenv "EE_HELP_OFF")
        (progn (ee-zap-help) 
		(setq ee-added-welcome " (ESC H for help)")))

  (message (concat "Welcome to Easy-edit " ee-VERSION " by Larry Broda " 
		ee-added-welcome))
)
